import unittest
from unittest.mock import Mock, patch, mock_open
from pathlib import Path
from django.core.management.base import CommandError
from django.test import TestCase
from django_spellbook.management.commands.processing.url_view_generator import URLViewGenerator
from django_spellbook.markdown.context import SpellbookContext
from django_spellbook.management.commands.processing.file_processor import ProcessedFile
from django_spellbook.utils import get_clean_url


class TestURLViewGenerator(TestCase):
    def setUp(self):
        self.generator = URLViewGenerator('test_app', '/test/path')
        self.mock_context = Mock(spec=SpellbookContext)
        self.mock_context.__dict__ = {'title': 'Test', 'toc': {}}

        # Create a sample processed file
        self.processed_file = ProcessedFile(
            original_path=Path('/test/file.md'),
            html_content='<h1>Test</h1>',
            template_path=Path('/test/template.html'),
            relative_url='test/page',
            context=self.mock_context
        )

    def test_initialization(self):
        """Test URLViewGenerator initialization."""
        self.assertEqual(self.generator.content_app, 'test_app')
        self.assertEqual(self.generator.content_dir_path, '/test/path')
        self.assertTrue(hasattr(self.generator, 'spellbook_dir'))

    @patch('os.path.exists')
    @patch('builtins.open', new_callable=mock_open)
    def test_ensure_urls_views_files_creation(self, mock_file, mock_exists):
        """Test creation of URLs and views files."""
        mock_exists.return_value = False

        self.generator._ensure_urls_views_files()

        self.assertEqual(mock_file.call_count, 2)
        mock_file.assert_any_call(
            f"{self.generator.spellbook_dir}/urls.py", 'w')
        mock_file.assert_any_call(
            f"{self.generator.spellbook_dir}/views.py", 'w')

    def test_generate_view_name(self):
        """Test view name generation from URL pattern."""
        url_pattern = 'docs/getting-started/index'
        expected = 'view_docs_getting_started_index'

        result = self.generator._generate_view_name(url_pattern)

        self.assertEqual(result, expected)

    def test_get_template_path(self):
        """Test template path generation."""
        relative_url = 'docs/index'
        expected = 'test_app/spellbook_md/docs/index.html'

        result = self.generator._get_template_path(relative_url)

        self.assertEqual(result, expected)

    def test_prepare_context_dict(self):
        """Test context dictionary preparation."""
        context = Mock(spec=SpellbookContext)
        context.__dict__ = {
            'title': 'Test',
            'date': '2023-01-01',
            'toc': {'some': 'data'}
        }

        result = self.generator._prepare_context_dict(context)

        self.assertNotIn('toc', result)
        self.assertEqual(result['title'], 'Test')

    @patch('builtins.open', new_callable=mock_open)
    def test_write_urls(self, mock_file):
        """Test writing URL patterns to file."""
        urls = ["path('test', views.view_test, name='view_test')"]

        self.generator._write_urls(urls)

        mock_file.assert_called_once()
        written_content = mock_file().write.call_args[0][0]
        self.assertIn('urlpatterns = [', written_content)
        self.assertIn("path('test'", written_content)

    @patch('builtins.open', new_callable=mock_open)
    def test_write_urls_with_dashes(self, mock_file):
        """Test writing URL patterns to file with dashes in URLs."""
        urls = ["path('--test', views.view_test, name='view_test')"]

        self.generator._write_urls(urls)

        mock_file.assert_called_once()
        written_content = mock_file().write.call_args[0][0]
        self.assertIn('urlpatterns = [', written_content)
        self.assertIn("path('test'", written_content)

    @patch('builtins.open', new_callable=mock_open)
    def test_write_urls_with_dashes_in_multiple_parts(self, mock_file):
        """Test writing URL patterns to file with dashes in multiple parts."""
        urls = [
            "path('--test-name/--test-name-2', views.view_test, name='view_test')",
        ]

        self.generator._write_urls(urls)

        mock_file.assert_called_once()
        written_content = mock_file().write.call_args[0][0]
        self.assertIn('urlpatterns = [', written_content)
        self.assertIn("path('test-name/test-name-2'", written_content)

    @patch('builtins.open', new_callable=mock_open)
    def test_write_views(self, mock_file):
        """Test writing view functions to file."""
        views = [
            "def view_test(request):\n    return render(request, 'test.html', {})"]
        toc = {'test': {'title': 'Test'}}

        self.generator._write_views(views, toc)

        mock_file.assert_called_once()
        written_content = mock_file().write.call_args[0][0]
        self.assertIn('from django.shortcuts import render', written_content)
        self.assertIn('TOC = ', written_content)

    @patch.object(URLViewGenerator, '_write_urls')
    @patch.object(URLViewGenerator, '_write_views')
    def test_generate_urls_and_views(self, mock_write_views, mock_write_urls):
        """Test full URL and view generation process."""
        processed_file = ProcessedFile(
            original_path=Path('/test/file.md'),
            html_content='<h1>Test</h1>',
            template_path=Path('/test/template.html'),
            relative_url='test',
            context=self.mock_context
        )

        self.generator.generate_urls_and_views([processed_file], {})

        mock_write_urls.assert_called_once()
        mock_write_views.assert_called_once()

    def test_create_file_if_not_exists_error(self):
        """Test file creation error handling"""
        with patch('os.path.exists', return_value=False):
            with patch('builtins.open', mock_open()) as mock_file:
                mock_file.side_effect = IOError("Permission denied")

                with self.assertRaises(CommandError) as context:
                    self.generator._create_file_if_not_exists(
                        'test.py', 'content')

                self.assertIn("Failed to create test.py",
                              str(context.exception))

    def test_generate_urls_and_views_error(self):
        """Test error handling in generate_urls_and_views"""
        with patch.object(self.generator, '_generate_url_data') as mock_generate:
            mock_generate.side_effect = Exception("Generation error")

            with self.assertRaises(CommandError) as context:
                self.generator.generate_urls_and_views(
                    [self.processed_file], {})

            self.assertIn("Failed to generate URLs and views",
                          str(context.exception))

    def test_write_urls_error(self):
        """Test error handling in _write_urls"""
        with patch.object(self.generator, '_write_file') as mock_write:
            mock_write.side_effect = Exception("Write error")

            with self.assertRaises(CommandError) as context:
                self.generator._write_urls(['test_url'])

            self.assertIn("Failed to write URLs file", str(context.exception))

    def test_write_views_error(self):
        """Test error handling in _write_views"""
        with patch.object(self.generator, '_write_file') as mock_write:
            mock_write.side_effect = Exception("Write error")

            with self.assertRaises(CommandError) as context:
                self.generator._write_views(['test_view'], {})

            self.assertIn("Failed to write views file", str(context.exception))

    def test_write_file_error(self):
        """Test error handling in _write_file"""
        with patch('builtins.open', mock_open()) as mock_file:
            mock_file.side_effect = IOError("Write permission denied")

            with self.assertRaises(CommandError) as context:
                self.generator._write_file('test.py', 'content')

            self.assertIn("Failed to write test.py", str(context.exception))

    @patch('os.path.exists')
    @patch('os.path.abspath')
    def test_ensure_urls_views_files_multiple_errors(self, mock_abspath, mock_exists):
        """Test handling of multiple file creation errors"""
        mock_abspath.return_value = '/test/path'
        mock_exists.return_value = False

        with patch('builtins.open', mock_open()) as mock_file:
            mock_file.side_effect = IOError("Multiple errors")

            with self.assertRaises(CommandError) as context:
                self.generator._ensure_urls_views_files()

            self.assertIn("Failed to create", str(context.exception))

    def test_generate_url_data_error(self):
        """Test error handling in _generate_url_data with invalid processed file"""
        invalid_processed_file = Mock(spec=ProcessedFile)
        invalid_processed_file.relative_url = None  # This should cause an error

        with self.assertRaises(AttributeError):
            self.generator._generate_url_data(invalid_processed_file)

    def test_prepare_context_dict_error(self):
        """Test error handling in _prepare_context_dict with invalid context"""
        invalid_context = Mock(spec=SpellbookContext)
        invalid_context.__dict__ = {'toc': {}, 'date': 'invalid-date'}

        # Should not raise an exception but handle the invalid date gracefully
        result = self.generator._prepare_context_dict(invalid_context)
        self.assertNotIn('toc', result)

    @patch('os.path.abspath')
    def test_get_spellbook_dir_error(self, mock_abspath):
        """Test error handling in _get_spellbook_dir"""
        mock_abspath.side_effect = Exception("Path error")

        with self.assertRaises(Exception) as context:
            self.generator._get_spellbook_dir()

        self.assertIn("Path error", str(context.exception))

    def test_generate_views_file_content_error_with_invalid_toc(self):
        """Test error handling in _generate_views_file_content with invalid TOC types"""
        # Test with a TOC that can't be string-formatted
        class CustomObject:
            def __str__(self):
                raise ValueError("Cannot convert to string")

        invalid_toc = CustomObject()

        with self.assertRaises(ValueError):
            self.generator._generate_views_file_content(
                ['test_view'], invalid_toc)

    def test_generate_views_file_content_with_invalid_views(self):
        """Test _generate_views_file_content with invalid views list"""
        # Test with invalid views content
        invalid_views = [None, 123, object()]

        with self.assertRaises(TypeError):
            self.generator._generate_views_file_content(invalid_views, {})

    def test_generate_view_name_with_dashes(self):
        """Test view name generation with dashes in URL pattern."""
        url_pattern = '--view-name'
        expected = 'view___view_name'

        result = self.generator._generate_view_name(url_pattern)

        self.assertEqual(result, expected)

    def test_get_clean_url(self):
        """Test URL cleaning"""
        url = '--test-url/--test-url-2/---test-url-3'
        expected = 'test-url/test-url-2/test-url-3'

        result = get_clean_url(url)

        self.assertEqual(result, expected)

    @patch('os.path.join')
    def test_get_template_path_error(self, mock_join):
        """Test error handling in _get_template_path"""
        mock_join.side_effect = Exception("Path join error")

        with self.assertRaises(Exception):
            self.generator._get_template_path("test/url")

    def test_generate_view_name_error(self):
        """Test error handling in _generate_view_name with invalid input"""
        invalid_url = None

        with self.assertRaises(AttributeError):
            self.generator._generate_view_name(invalid_url)



# tests/conftest.py
import django
from django.conf import settings


def pytest_configure():
    settings.configure(
        DEBUG=True,
        DATABASES={
            'default': {
                'ENGINE': 'django.db.backends.sqlite3',
                'NAME': ':memory:',
            }
        },
        INSTALLED_APPS=[
            'django.contrib.contenttypes',
            'django.contrib.auth',
            'django_spellbook',
        ],
        MIDDLEWARE=[],
        ROOT_URLCONF='',
        TEMPLATES=[{
            'BACKEND': 'django.template.backends.django.DjangoTemplates',
            'APP_DIRS': True,
        }],
        SECRET_KEY='test-key',
        SPELLBOOK_MD_PATH='/test/path',
        SPELLBOOK_CONTENT_APP='test_app',
    )
    django.setup()



from django.test import TestCase

from django_spellbook.utils import remove_leading_dash, titlefy


class TestUtils(TestCase):
    def test_remove_leading_dash(self):
        """Test removing leading dashes from a string"""
        result = remove_leading_dash('--test')
        self.assertEqual(result, 'test')

    def test_titlefy(self):
        """Test titlefy function"""
        result = titlefy('test-page')
        self.assertEqual(result, 'Test Page')
        result = titlefy('this-is-a-test-page')
        self.assertEqual(result, 'This is a Test Page')

    def test_titlefy_without_dashes(self):
        """Test titlefy function without dashes"""
        result = titlefy('test page')
        self.assertEqual(result, 'Test Page')



from typing import Dict, Type, Optional
import importlib
import logging
from django.apps import apps
from .base import BasicSpellBlock
from .exceptions import BlockRegistrationError

logger = logging.getLogger(__name__)


class SpellBlockRegistry:
    """
    Registry for spell blocks that handles registration and retrieval of block classes.

    This class provides a central registry for all spell blocks in the application,
    ensuring unique naming and proper inheritance from BasicSpellBlock.
    """
    _registry: Dict[str, Type[BasicSpellBlock]] = {}
    _discovery_in_progress = False

    @classmethod
    def register(cls, name: Optional[str] = None):
        """
        Decorator to register a spell block class in the registry.

        Args:
            name (Optional[str]): Optional custom name for the block.
                If not provided, will use the block class's 'name' attribute.

        Returns:
            callable: Decorator function that registers the block class.

        Raises:
            BlockRegistrationError: If registration fails due to:
                - Invalid inheritance
                - Missing block name
                - Name conflicts
                - Other registration errors
        """
        def decorator(block_class: Type[BasicSpellBlock]):
            try:
                # Validate the block class
                if not issubclass(block_class, BasicSpellBlock):
                    raise BlockRegistrationError(
                        f"Block class {block_class.__name__} must inherit from BasicSpellBlock"
                    )

                # Get the block name
                block_name = name or getattr(block_class, 'name', None)
                if not block_name:
                    raise BlockRegistrationError(
                        f"Block class {block_class.__name__} must have a name"
                    )

                # Check for name conflicts
                if block_name in cls._registry:
                    raise BlockRegistrationError(
                        f"Multiple blocks registered with name '{block_name}'"
                    )

                # Register the block
                cls._registry[block_name] = block_class
                logger.debug(f"Successfully registered block: {block_name}")
                return block_class

            except Exception as e:
                logger.error(
                    f"Error registering block {block_class.__name__}: {str(e)}")
                raise BlockRegistrationError(str(e))
        return decorator

    @classmethod
    def get_block(cls, name: str) -> Optional[Type[BasicSpellBlock]]:
        """
        Retrieve a registered block class by its name.

        Args:
            name (str): Name of the block to retrieve.

        Returns:
            Optional[Type[BasicSpellBlock]]: The registered block class if found, None otherwise.
        """
        return cls._registry.get(name)



import unittest
from unittest.mock import Mock, patch, mock_open
from pathlib import Path
import django
from django.conf import settings
from django.test import override_settings
from django_spellbook.management.commands.processing.file_processor import (
    MarkdownFileProcessor,
    MarkdownProcessingError,
)


def setup_django_settings():
    from django.conf import settings
    if not settings.configured:
        settings.configure(
            DEBUG=True,
            DATABASES={
                'default': {
                    'ENGINE': 'django.db.backends.sqlite3',
                    'NAME': ':memory:',
                }
            },
            INSTALLED_APPS=[
                'django_spellbook',
            ],
            MIDDLEWARE=[],
            ROOT_URLCONF=[],
            TEMPLATES=[],
            SECRET_KEY='test-key',
            SPELLBOOK_MD_PATH='/fake/path',
        )
    django.setup()


# Run settings configuration before tests
setup_django_settings()


class TestMarkdownFileProcessor(unittest.TestCase):
    def setUp(self):
        self.processor = MarkdownFileProcessor()

    @override_settings(SPELLBOOK_MD_PATH='/fake/path')
    def test_validate_and_get_path(self):
        """Test path validation and creation"""
        dirpath = "/test/path"
        filename = "test.md"

        result = self.processor._validate_and_get_path(dirpath, filename)

        self.assertIsInstance(result, Path)
        self.assertEqual(str(result), "/test/path/test.md")

    @override_settings(SPELLBOOK_MD_PATH='/fake/path')
    def test_validate_and_get_path_invalid(self):
        """Test path validation with invalid input"""
        dirpath = None
        filename = "test.md"

        with self.assertRaises(MarkdownProcessingError):
            self.processor._validate_and_get_path(dirpath, filename)

    @override_settings(SPELLBOOK_MD_PATH='/fake/path')
    @patch('builtins.open', new_callable=mock_open, read_data="# Test Content")
    def test_read_markdown_file(self, mock_file):
        """Test reading markdown file content"""
        test_path = Path("/test/path/test.md")

        content = self.processor._read_markdown_file(test_path)

        self.assertEqual(content, "# Test Content")
        mock_file.assert_called_once_with(test_path, 'r', encoding='utf-8')

    @override_settings(SPELLBOOK_MD_PATH='/fake/path')
    @patch('builtins.open')
    def test_read_markdown_file_error(self, mock_file):
        """Test reading markdown file with error"""
        mock_file.side_effect = IOError("File not found")
        test_path = Path("/test/path/test.md")

        with self.assertRaises(MarkdownProcessingError) as context:
            self.processor._read_markdown_file(test_path)

        self.assertIn("Error reading file", str(context.exception))

    @override_settings(SPELLBOOK_MD_PATH='/fake/path')
    @patch('django_spellbook.management.commands.processing.file_processor.FrontMatterParser')
    @patch('django_spellbook.management.commands.processing.file_processor.MarkdownParser')
    def test_process_markdown_content(self, mock_md_parser, mock_frontmatter):
        """Test markdown content processing"""
        # Setup mocks
        mock_frontmatter.return_value.raw_content = "# Test Content"
        mock_md_parser.return_value.get_html.return_value = "<h1>Test Content</h1>"

        md_text = "---\ntitle: Test\n---\n# Test Content"
        file_path = Path("/test/path/test.md")

        html_content, frontmatter = self.processor._process_markdown_content(
            md_text, file_path)

        self.assertEqual(html_content, "<h1>Test Content</h1>")
        mock_frontmatter.assert_called_once_with(md_text, file_path)

    @override_settings(SPELLBOOK_MD_PATH='/fake/path')
    def test_calculate_relative_url(self):
        """Test relative URL calculation"""
        file_path = Path("/fake/path/subfolder/test.md")

        relative_url = self.processor._calculate_relative_url(file_path)

        self.assertEqual(relative_url, "subfolder/test")

    @override_settings(SPELLBOOK_MD_PATH='/fake/path')
    @patch('django_spellbook.management.commands.spellbook_md.FrontMatterParser')
    def test_generate_file_metadata(self, mock_frontmatter):
        """Test file metadata generation"""
        # Setup
        file_path = Path("/fake/path/test.md")
        html_content = "<h1>Test</h1>"
        mock_context = Mock()
        mock_frontmatter.return_value.get_context.return_value = mock_context

        processed_content = (html_content, mock_frontmatter.return_value)

        # Test
        result_html, result_path, result_context = self.processor._generate_file_metadata(
            file_path, processed_content
        )

        # Assertions
        self.assertEqual(result_html, html_content)
        self.assertEqual(result_path, file_path)
        self.assertEqual(result_context, mock_context)

    @override_settings(SPELLBOOK_MD_PATH='/fake/path')
    @patch.object(MarkdownFileProcessor, '_validate_and_get_path')
    @patch.object(MarkdownFileProcessor, '_read_markdown_file')
    @patch.object(MarkdownFileProcessor, '_process_markdown_content')
    @patch.object(MarkdownFileProcessor, '_generate_file_metadata')
    def test_process_file_integration(self, mock_generate, mock_process,
                                      mock_read, mock_validate):
        """Test the full process_file integration"""
        # Setup mocks
        mock_validate.return_value = Path("/test/path/test.md")
        mock_read.return_value = "# Test Content"
        mock_process.return_value = ("<h1>Test Content</h1>", Mock())
        mock_generate.return_value = ("<h1>Test Content</h1>",
                                      Path("/test/path/test.md"),
                                      Mock())

        # Test
        result = self.processor.process_file(
            Path("/test/path"),
            "/test/path",
            "test.md",
            ["test"]
        )

        # Assertions
        self.assertIsNotNone(result)
        mock_validate.assert_called_once()
        mock_read.assert_called_once()
        mock_process.assert_called_once()
        mock_generate.assert_called_once()

    def test_process_file_general_exception(self):
        """Test process_file handling of unexpected exceptions"""
        with patch.object(self.processor, '_validate_and_get_path') as mock_validate:
            mock_validate.side_effect = Exception("Unexpected error")

            with self.assertRaises(MarkdownProcessingError) as context:
                self.processor.process_file(
                    Path("/test/path"),
                    "/test/path",
                    "test.md",
                    ["test"]
                )

            self.assertIn("Unexpected error processing test.md",
                          str(context.exception))

    def test_process_markdown_content_exception(self):
        """Test process_markdown_content error handling"""
        with patch('django_spellbook.management.commands.processing.file_processor.FrontMatterParser') as mock_frontmatter:
            mock_frontmatter.side_effect = Exception("Frontmatter error")

            with self.assertRaises(MarkdownProcessingError) as context:
                self.processor._process_markdown_content(
                    "# test", Path("/test/test.md"))

            self.assertIn("Error processing markdown", str(context.exception))

    def test_generate_file_metadata_exception(self):
        """Test generate_file_metadata error handling"""
        file_path = Path("/test/test.md")
        processed_content = (
            "<h1>Test</h1>", Mock(side_effect=Exception("Context error")))

        with self.assertRaises(MarkdownProcessingError) as context:
            self.processor._generate_file_metadata(
                file_path, processed_content)

        self.assertIn("Error generating metadata", str(context.exception))

    def test_calculate_relative_url_exception(self):
        """Test calculate_relative_url error handling"""
        file_path = Path("/invalid/path")

        with patch('pathlib.Path.relative_to') as mock_relative:
            mock_relative.side_effect = Exception("Path error")

            with self.assertRaises(MarkdownProcessingError) as context:
                self.processor._calculate_relative_url(file_path)

            self.assertIn("Error calculating relative URL",
                          str(context.exception))

    def test_process_file_markdown_processing_error(self):
        """Test process_file handling of MarkdownProcessingError"""
        with patch.object(self.processor, '_validate_and_get_path') as mock_validate:
            mock_validate.side_effect = MarkdownProcessingError("Test error")

            with self.assertRaises(MarkdownProcessingError) as context:
                self.processor.process_file(
                    Path("/test/path"),
                    "/test/path",
                    "test.md",
                    ["test"]
                )

            self.assertEqual(str(context.exception), "Test error")

    def test_generate_file_metadata_invalid_content(self):
        """Test generate_file_metadata with invalid processed content"""
        file_path = Path("/test/test.md")
        invalid_content = ("content",)  # Missing second element of tuple

        with self.assertRaises(MarkdownProcessingError) as context:
            self.processor._generate_file_metadata(file_path, invalid_content)

        self.assertIn("Error generating metadata", str(context.exception))

    def test_process_markdown_content_invalid_parser(self):
        """Test process_markdown_content with parser error"""
        # Create a mock parser instance
        mock_parser_instance = Mock()
        mock_parser_instance.get_html.side_effect = Exception("Parser error")

        # Create a mock parser class that returns our mock instance
        mock_parser_class = Mock(return_value=mock_parser_instance)

        # Temporarily replace the processor's parser with our mock
        original_parser = self.processor.parser
        self.processor.parser = mock_parser_class

        try:
            with self.assertRaises(MarkdownProcessingError) as context:
                self.processor._process_markdown_content(
                    "# test", Path("/test/test.md"))

            self.assertIn("Error processing markdown", str(context.exception))
        finally:
            # Restore the original parser
            self.processor.parser = original_parser



# tests/settings.py
import os

SECRET_KEY = 'test-key'
INSTALLED_APPS = [
    'django.contrib.contenttypes',
    'django.contrib.auth',
    'django_spellbook',
]

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': ':memory:',
    }
}

MIDDLEWARE = []

TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'DIRS': [
            os.path.join(os.path.dirname(__file__), 'templates'),
        ],
        'APP_DIRS': True,
        'OPTIONS': {
            'context_processors': [
                'django.template.context_processors.debug',
                'django.template.context_processors.request',
            ],
        },
    },
]



from typing import Set

from .base import BasicSpellBlock
from .registry import SpellBlockRegistry

# Export what we want available at the package level
__all__ = ['BasicSpellBlock', 'SpellBlockRegistry']




'''import re
from markdown.blockprocessors import BlockProcessor
from markdown.extensions import Extension
from xml.etree import ElementTree


class CodeBlockProcessor(BlockProcessor):
    RE_CODE_BLOCK = re.compile(r'```(\w+)?\s*([\s\S]+?)\s*```')

    def test(self, parent, block):
        # More flexible test that handles leading whitespace
        return '```' in block.strip()

    def run(self, parent, blocks):
        block = blocks.pop(0)
        m = self.RE_CODE_BLOCK.search(block)  # Use search instead of match

        if not m:
            return False

        language = m.group(1)
        content = m.group(2)

        # Create pre and code elements
        pre_elem = ElementTree.SubElement(parent, 'pre')
        code_elem = ElementTree.SubElement(pre_elem, 'code')

        if language:
            code_elem.set('data-language', language)
            code_elem.set('class', f'language-{language}')

        # Preserve whitespace but trim extra newlines
        code_elem.text = content.strip()

        # Handle remaining content
        remaining = block[m.end():].lstrip()
        if remaining:
            blocks.insert(0, remaining)

        return True


class CodeBlockExtension(Extension):
    def extendMarkdown(self, md):
        md.parser.blockprocessors.register(
            CodeBlockProcessor(md.parser), 'code_block', 180)


def makeExtension(**kwargs):
    return CodeBlockExtension(**kwargs)
'''
